'use strict'
var iUtil = require('./util')
var extend = require('extend')
var colors = require('colors')
var controller = require('./controller')
var mime =require('mime')
var Mock =require('./mock')
var _util = require('./util')
var doc = require('./doc')
var ajax = function (url, settings) {
    var $ = require('../index')
    var handler
    var sContentType
    settings = settings || {}
     // 通过判断 url 提供 ajax( url [, settings ] ) 参数支持
    switch (typeof url) {
        // ajax('/ajax/url/', settings?)
        case 'string':
            settings.url = url
        break
        // ajax(settings)
        case 'object':
            settings = url
        break
        default:
    }
    settings.url = iUtil.checkUrl(settings.url)
    // url 错误时会返回 false
    if (!settings.url){
        return false
    }

    settings = extend(true, {} ,$.config.ajaxSettings ,settings)

    // post => POST , get => GET , all => all
    settings.type = settings.type.toUpperCase()
    if (settings.type !== 'ALL' && settings.type !== 'GET' && settings.type !== 'POST' && settings.type !== 'PUT' && settings.type !== 'DELETE') {
        console.log('----------------------------------------'.grey)
        console.log('settings.type: '.red + settings.type + ' Parameter error !'.red)
        console.log('----------------------------------------'.grey)
        throw "settings.type allow only 'all' 'get' 'post' 'put' 'delete' !"
    }

    if (settings.dataType === 'jsonp') {
        sContentType = 'text/javascript'
    }
    else {
        // json ==> application/json , text ==> text/html
        sContentType = mime.lookup(settings.dataType)
    }

    settings.successStatusCode = settings.statusCode[0]
    settings.errorStatusCode = settings.statusCode[1]

    
    $.handle(settings.url, 'ajax', settings.type, function (req, res) {

        if (settings.timeout === 'ever') {
            return false
        }
        // 通过cookie 控制ajax成功失败

        _util.namespace('ajax*' + req.path, req.cookies.fms, '*')
        
        var oThisControlCookie = req.cookies.fms['ajax'][req.path]
        oThisControlCookie.type = oThisControlCookie.type || 'success'

        if (req.cookies._) {
            req.cookies._ = oThisControlCookie
        }
        
        req.cookies.fms_this = oThisControlCookie

        var responseResult
        var iStatusCode
        
        // 默认成功

        var sResultType = oThisControlCookie.type

        // 根据 GET 参数 `_` 控制 AJAX 结果
        // Ref: https://github.com/nimojs/fms/issues/5

        var control_
        if (settings.type === 'GET') {
            control_ = req.query._
        }
        else {
            // POST PUT DELETE
            control_ = req.query.body
        }
        switch (control_) {
            case "1":
            sResultType = 'success'
            break
            case "0":
            sResultType = 'error'
            break
            case undefined:
            break
            default:
            sResultType = control_
        }
        

        if(sResultType === 'success') {
            responseResult = settings.success
            iStatusCode = settings.statusCode[0]
        }
        else if(sResultType === 'error') {
            responseResult = settings.error
            iStatusCode = settings.statusCode[1]
        }
        // Ref: https://github.com/nimojs/fms/issues/8
        else {
            iStatusCode = 200
            if (typeof settings.res[sResultType] !== 'undefined') {
                responseResult = settings.res[sResultType]
            }
            else {
                res.status(415)
                res.send('Not find "' + settings.url + '" settings res.' + sResultType )
                return
            }
        }
        
        // 递归将 settings.success 和 settings.error 转换为字符串或 function 默认返回的 undefined
        (function translateResponse () {
            switch (typeof responseResult) {
            case 'string':
            break
            case 'function':
                responseResult = responseResult(req, res)
                // responseResult could be Number Object
                translateResponse()
            break
            case 'object':                
                responseResult = JSON.stringify(Mock.mock(responseResult))
            break
            case 'number':
                responseResult = String(responseResult)
            break
            default:
            }
        })()
        /*
        responseResult = function(res, res){
            res.send('message')
        }
        responseResult() === undefined
        如果 responseResult 是 undefined 则表示已经使用了 res.send 之类方法 响应了请求
        @todo
        文档：如果没有 res.send 没有return undefined 以外的值则会一直无响应
        */
        if (responseResult !== undefined) {
            res.status(iStatusCode)

            res.set({
                "Content-Type": sContentType
            })
            setTimeout(function() {
                if (settings.dataType === 'jsonp') {
                    res.jsonp(responseResult)
                } else {
                    res.send(responseResult)
                }
            },
            settings.timeout)
        }
    })
    
    var oCallbackHandler = {
        done: function (successOpt) {
            settings.success = successOpt
            return this
        },
        fail: function (errorOpt) {
            settings.error = errorOpt
            return this
        },
        always: function (alwaysOpt) {
            settings.success = alwaysOpt
            settings.error = alwaysOpt
            settings.isAlways = true
            return this
        },
        doc: function (docOpt) {
            settings.doc = docOpt || settings.doc
            return this
        }
    }
    // 增加文档
    doc._data.push({
        type: 'ajax',
        settings: settings
    })
    
    // 兼容低版本 jqXHR API
    oCallbackHandler.success = oCallbackHandler.done
    // Ref: https://github.com/nimojs/fms/issues/25
    oCallbackHandler.ok = oCallbackHandler.done
    oCallbackHandler.error = oCallbackHandler.fail
    oCallbackHandler.all = oCallbackHandler.always
    $._data.ajaxs.push({
        title: settings.title || '',
        url: settings.url
    })
    return oCallbackHandler
}
function get (url, success, error) {
    if (typeof url === 'object') {
        var settings = url
        settings.type = 'get'
        return ajax(settings)
    }
    else {
        var settings = {
            url: url,
            type: 'GET'
        }
        return ajax(settings)
    }
}
function post (url, success, error) {
    if (typeof url === 'object') {
        var settings = url
        settings.type = 'post'
        return ajax(settings)
    }
    else {
        var settings = {
            url: url,
            type: 'POST'
        }
        return ajax(settings)
    }
}
module.exports = {
    ajax: ajax,
    get: get,
    post: post
}