'use strict'
var config = {

    // 新手参数 Start
    // 端口 - Port: http://127.0.0.1:3000/
    // Example: 3000 or 8888 or 3000
    port: 3000,

    // 默认为 app.js 所在目录下的所有文件均为静态资源文件
    static: process.cwd(),

    // 首页中 HTML list 读取的目录 - Home page HTML list read directory
    html: [],

    // markdown/html 文档的保存地址
    docPath: {
        html: '/docs/data-api.html',
        markdown: '/docs/data-api.md'
    },

    // 新手参数 End

    // express 中的 connect，如不熟悉请无视
    connect: function (req, res ,next) {
        next();
    },

    // 访问密码 - Access to the password
    // Example:  false or "nodeajax"
    // 参数是 false 则不需要密码 - If parameter is false, do not need a password
    password: false,

    // 因为与 combo 不好共存，先注释
    // // URL重定向 - Rewrite rule
    // urlRewrite: [
    //     // ['/', '/html/index.html']
    // ],

    ajaxSettings: {
        type: 'get',
        // // 请求参数
        // data: {
        //     id: 1
        // }
        success: {
            status: 'success'
        },
        error: {
            status: 'error',
            msg: 'Error detail'
        },
        doc: function() {/*!
`AJAX` {{title}} 
---------
- url: [{{url}}]({{url}})
- type: `{{type}}`

{{#if request}}
#### Request:
{{{request}}}
{{/if}}
#### Response:
{{#if success}}
**success:**  
```js
{{success}}
```
{{/if}}
{{#unless isAlways}}
{{#if error}}
**error:**  
```js
{{error}}
```
{{/if}}
{{/unless}}
        */},
        statusCode: [200, 200],
        timeout: 300,
        dataType: 'html'
    },
    
    errorPage: '/html/404.html',

    // 静态资源替换 - Static write replace
    staticWriteReplace: function (file, req) {
        // 一定要同步操作 existsSync readFileSync
        // file = {
        //     pathname: pathname ( example/index.html )
        //     dirname: dirname (example/)
        //     data: filedata ( <!DOCTYPE html> <html lang="en"> <head> ... </html> )
        // }

        // 显示特定注释中的代码
        // <!--nodeajax
        // <strong>text</strong>
        // nodeajax-->
        // 将会被替换为如下代码
        // <strong>text</strong>
        file.data = file.data.replace(/<\!--nodeajax([\s\S]*)?nodeajax-->/gm, '$1');

        // 原生PHP include 语法支持
        // <?php include "common/header"; ?>
        // content
        // <?php include "common/footer"; ?>
        file.data = file.data.replace(/<\?php (include|require) ["]([^"]+)"; \?>/g, function () {
            var sPath = arguments[2],
            fs = require('fs');
            var sPath = file.dirname + sPath
            if (!fs.existsSync(sPath)) {
                return '<strong style="color:#333;background:#f2dede;">ENOENT, no such file ' + sPath + '</strong>'; 
            } else {
                try {
                    return fs.readFileSync(sPath, 'utf-8');
                } catch (err) {
                    return err.message;
                }
            }
        })
        
    },
    pageSettings: {
        data: {
            /*
            DEV:true,
            APP_PATH: '/',
            USER: {
                id: 23,
                name: 'nimo',
                email: 'nimo.jser@gmail.com',
                avatar: 'https://avatars1.githubusercontent.com/u/3949015?v=3&s=50'
            }
            */
        },
        render:function() {},
        doc: function() {/*!
`Page` {{title}}
---------
- url: [{{url}}]({{url}})
- template: `{{template}}`

{{#if request}}
#### Request:
{{request}}
{{/if}}
#### Data:
{{#if data}}
**success:**  
```js
{{data}}
```
{{/if}}

        */}
    },
    smarty: {
        includePath: '',
        plugin: {
            json: function (settings) {
                return JSON.stringify(settings.data)
            },
            date: function (settings) {
                var time = settings['time']
                var format = settings['format']
                if (!time) {
                    time = new Date().getTime()
                }
                if (typeof time === 'string') {
                    time = parseInt(time, 10)
                }

                if (!format) {
                    format = 'Y-m-d H:i:s'
                }        
                return phpdate(format, time)
            }
        },
        // 在 page 文档中过滤这些选项
        docDataFilter: ['DEV','APP_PATH'],
        // request 用于向数据源提供 http 相关信息。全局性，每次渲染都会执行
        // 渲染执行顺序:1
        request: function (data, req) {
            /*
            不允许重写 data ：
            data = {}
            */
            data.PAGE_URL = req.url;
            data.PAGE_PATH = req.path;
            data.METHOD = req.method;
            data.GET = req.query;
            data.POST = req.body;
        }
    }
}
module.exports = config