'use strict'
var fs = require('fs')
var _ = require('underscore')
var readdir = require('readdir')
var eachAsync = require('each-async')
var qrcode = require('iqrcode')
var fse = require('fs-extra')
var mime = require('mime')
var Handlebars = require('handlebars')

var _util = require('./util')

var controller  = {
    handlers: {
        /*
        '/a/': {
            default: function (req, res) {
            
            }
        },
        '/b/': {
            ajax: {
                "GET":fn,
                "POST":fn,
                "PUT":fn,
                "DELETE":fn,
                "ALL":fn
            }
        },
        '/c/': {
            page: fn
        }
        */
    },
    method: {
        
    }
}
var regexp = {
    titleTag: /<\s*title\s*>([^<]*)<\/\s*title\s*>/
}

controller.handle = function (url, type, method, fn) {
    var $ = require('../index')
    if (typeof type === 'function') {
        fn = type
        type = 'default'
    }

    controller.handlers[url] = controller.handlers[url] || {}
    
    if (type === 'ajax') {
        _util.namespace(url + '*' + type , controller.handlers, '*')
        if (controller.handlers[url].ajax[method]) {
            console.log('------- Trace -------'.grey)
            console.trace('$.' + type + ' ' + url + method + ' is exist!')
            console.log('--------------------'.grey)
            console.log('$.' + type + ' ' + method + ' ' + url.yellow + ' is exist!'.red)
            console.log('--------------------'.grey)
            throw 'ajax is exist!'
        }
    }
    else if (controller.handlers[url][type]) {
        console.log('------- Trace -------'.grey)
        console.trace('$.' + type + ' ' + url + ' is exist!')
        console.log('--------------------'.grey)
        console.log('$.' + type + ' ' + url.yellow + ' is exist!'.red)
        console.log('--------------------'.grey)
        throw 'url is exist!'
    }

    controller.handlers[url][type] = controller.handlers[url][type] || {}
    
    if (method === 'all') {
        controller.handlers[url][type] = fn
    }
    else {
        controller.handlers[url][type][method] = fn
    }
}


controller.handle('_password', 'default', 'all', function (req, res) {
    var $ = require('../index')
    var sForm = $.heredoc(function () {/*!
        <form method="post">
            password:<input  autofocus="autofocus" type="password" name="password" />
            <button type="submit">Submit</button>
        </form>
    */})
    if (req.method === 'GET') {
        res.send(sForm)
    }
    else {
        if (req.body.password === $.config.password) {
            res.cookie('password', $.config.password, {
                path: '/',
                maxAge: 31536000
            })
            res.redirect(301, req.originalUrl)
        }
        else {
            res.send(sForm + 'Password error!')
        }
    }
})

var staticTemplate
controller.handle('_static', 'default', 'all', function (req, res) {
    var $ = require('../index')
    var filepath = $.config.static + req.path
    var notfind = function () {
        res.status(404).send('not find')
    }
    fs.stat(filepath, function (err, stat) {
        if (err) {notfind();return}
        // 输出目录列表
        if (stat.isDirectory()) {
            readdir.read(filepath, readdir.INCLUDE_DIRECTORIES + readdir.NON_RECURSIVE, function (err, filesArray) {
                if (err) {notfind();return}
                fs.readFile('./handlebars/static.hbs', 'utf-8', function (err, source) {
                    if (err) {notfind();return}
                    staticTemplate = staticTemplate || Handlebars.compile(source)

                    var path = req.path
                    if (!/\/$/.test(path)) {
                        path = path + '/'
                    }
                    var html = staticTemplate({
                        title: req.path,
                        path: path,
                        list: filesArray
                    })
                    res.send(html)
                })
            })
        }
        // 输出文件
        else{
            fs.readFile(filepath, function (err, data) {
                if (err) {notfind();return}
                var contentType = mime.lookup(filepath)
                res.set({
                    'Content-Type': contentType
                })
                if (contentType === 'text/html' || contentType === 'text/css' || contentType === 'text/javascript') {
                    var file = {
                        pathname: filepath,
                        dirname: filepath.replace(/[^\/]+$/, ''),
                        data: data.toString()
                    }
                    $.config.staticWriteReplace(file, req)
                    res.send(file.data)
                }
                else {
                    res.send(data)
                }
            })
        }
    })
    
})

var dashboardHandle = function (req, res) {
    var $ = require('../index')
    var type = req.query.type
    switch (type) {
        case 'ajaxlist':
        res.json($._data.ajaxs)
        break
        case 'doc':
        res.send($._data.doc_html)
        break
        case 'config':
        res.json({
            docPath: $.config.docPath
        })
        break
        case 'pages':
        var list = [{
            "files":$._data.pages
        }]
        /*
        list 返回值格式：
        [
            {
                "title": "/html/",
                "filepath": [
                    [
                        "第一级",
                        "1.html"
                    ],
                    [
                        "第二级",
                        "some/1.html"
                    ]
                ]
            },
            {
                "title": "/",
                "filepath": [
                    [
                        " ",
                        "data-api.html"
                    ]
                ]
            }
        ]
        */

        // $.config.html = ["/", "html"]
        eachAsync($.config.html, function (dirname, index, nextEachDir) {
            var filter
            if (dirname === '/') {
                filter = ['*.html', '*.htm']
            }
            else {
                if (dirname.charAt(0) !== '/') {
                    dirname = '/' + dirname
                }
                if (!/\/$/.test(dirname)){
                    dirname = dirname + '/'
                }
                filter = ['**.html', '**.htm']
            }
            readdir.read( $.config.static + dirname, filter, readdir.INCLUDE_DIRECTORIES, function (err, filesArray) {
                if (err) {
                    if (err.code === 'ENOENT') {
                        list.push({
                            title: dirname,
                            files: []
                        })
                        nextEachDir()
                        return
                    }
                    console.log(err)
                }                
                // filesArray = ['index.html', 'demo.html']
                eachAsync(filesArray, function (filename, index ,next) {
                    var filepath = $.config.static + dirname  + filename
                    fs.readFile(filepath, 'utf-8', function (err, data) {
                        if (err) console.log(err)                            
                        var aPageTitle = data.match(regexp.titleTag)
                        var sTitle = 'not title'
                        if (aPageTitle) {
                            sTitle = aPageTitle[1]
                        }
                        // index.html = ['首页', 'index.html']
                        filesArray[index] = {
                            title: sTitle,
                            url: dirname + filename,
                            filename: filename.replace(/.*\/(.*)$/, '$1')
                        }
                        next()
                    })
                }, function () {
                    list.push({
                        title: dirname,
                        files: filesArray
                    })
                    // 遍历下一个目录
                    nextEachDir()
                })

            })

        }, function (err) {
        // 完成静态 HTML 的遍历
            if (err) console.log(err)
            res.json(list)        
        })
    

        break
        case 'qrcode':
        var text = req.query.text
        res.send(qrcode.toDataURL(text, 4))
        break
        case 'fms.css':
        fs.readFile(__dirname + '/static/css/fms.css', function (err, data) {
            if (err) throw err
            res.set({
                "Content-Type": 'text/css'
            })
            res.send(data.toString())
        })
        break
        case 'fms.js':
        fs.readFile(__dirname + '/static/js/fms.js', function (err, data) {
            if (err) throw err
            res.set({
                "Content-Type": 'application/x-javascript'
            })
            res.send(data.toString())
        })
        break
        default:
        fs.readFile(__dirname + '/static/dashboard.html', function (err, data) {
            if (err) throw err
            res.set({
                "Content-Type": 'text/html'
            })
            res.send(data.toString())
        })
    }
}

// 默认首页就是 dashboard ，如果想要修改首页就通过 urlRewrite 或者 $.ajax $.page $.handle
controller.handle('/', 'default', 'all', dashboardHandle)
// "//" 是 dashboard 的快捷访问地址
controller.handle("//", 'default', 'all',  dashboardHandle)
// "/fms/" 是一直存在的 dashboard 页面
controller.handle('/fms/', 'default', 'all', dashboardHandle)

module.exports = controller